--liquibase formatted sql

-- ============================================================================
--changeset hs-office-partner-rbac-OBJECT:1 endDelimiter:--//
-- ----------------------------------------------------------------------------
call generateRelatedRbacObject('hs_office_partner');
--//


-- ============================================================================
--changeset hs-office-partner-rbac-ROLES-CREATION:1 endDelimiter:--//
-- ----------------------------------------------------------------------------

/*
    Creates and updates the roles and their assignments for partner entities.
 */

create or replace function hsOfficePartnerRbacRolesTrigger()
    returns trigger
    language plpgsql
    strict as $$
declare
    partnerUuid          uuid default new.uuid;
    partnerDetailsUuid   uuid default new.detailsUuid;
    oldPartnerRel        hs_office_relationship;
    newPartnerRel        hs_office_relationship;

begin
    call enterTriggerForObjectUuid(NEW.uuid);

    select * from hs_office_relationship as r where r.uuid = NEW.partnerroleuuid into newPartnerRel;

    if TG_OP = 'INSERT' then

        -- Permissions and Grants for Partner

        call grantPermissionsToRole(
                getRoleId(hsOfficeRelationshipOwner(newPartnerRel)),
                createPermissions(partnerUuid, array ['DELETE'])
             );

        call grantPermissionsToRole(
                getRoleId(hsOfficeRelationshipAdmin(newPartnerRel)),
                createPermissions(partnerUuid, array ['UPDATE'])
             );

        call grantPermissionsToRole(
                getRoleId(hsOfficeRelationshipTenant(newPartnerRel)),
                createPermissions(partnerUuid, array ['SELECT'])
             );

        -- Permissions and Grants for PartnerDetails

        call grantPermissionsToRole(
                getRoleId(hsOfficeRelationshipOwner(newPartnerRel)),
                createPermissions(partnerDetailsUuid, array ['DELETE'])
             );

        call grantPermissionsToRole(
                getRoleId(hsOfficeRelationshipAdmin(newPartnerRel)),
                createPermissions(partnerDetailsUuid, array ['UPDATE'])
             );

        call grantPermissionsToRole(
            -- Yes, here hsOfficePartnerAGENT is used, not hsOfficeRelationshipTENANT.
            -- Do NOT grant view permission on partner-details to hsOfficeRelationshipTENANT!
            -- Otherwise package-admins etc. would be able to read the data.
                getRoleId(hsOfficeRelationshipAgent(newPartnerRel)),
                createPermissions(partnerDetailsUuid, array ['SELECT'])
             );


    elsif TG_OP = 'UPDATE' then

        if OLD.partnerRoleUuid <> NEW.partnerRoleUuid then
            select * from hs_office_relationship as r where r.uuid = OLD.partnerRoleUuid into oldPartnerRel;

            -- Revokes from Partner

            call revokePermissionFromRole(
                    findPermissionId(partnerUuid, 'SELECT'),
                    hsOfficeRelationshipTenant(oldPartnerRel)
                 );

            --             call revokePermissionFromRole(
            --                     findPermissionId(partnerUuid, 'edit'),
            --                     hsOfficeRelationshipAdmin(oldPartnerRel)
            --                  );
            --
            --             call revokePermissionFromRole(
            --                     findPermissionId(partnerUuid, '*'),
            --                     hsOfficeRelationshipOwner(oldPartnerRel)
            --                  );

            -- Grants for Partner

            call grantPermissionsToRole(
                    getRoleId(hsOfficeRelationshipOwner(newPartnerRel)),
                    array[findPermissionId(partnerUuid, 'DELETE')]
                 );

            call grantPermissionsToRole(
                    getRoleId(hsOfficeRelationshipAdmin(newPartnerRel)),
                    array[findPermissionId(partnerUuid, 'UPDATE')]
                 );

            call grantPermissionsToRole(
                    getRoleId(hsOfficeRelationshipTenant(newPartnerRel)),
                    array[findPermissionId(partnerUuid, 'SELECT')]
                 );

            -- Revokes from PartnerDetails

            --             call revokePermissionFromRole(
            --                     findPermissionId(partnerDetailsUuid, 'SELECT'),
            --                     hsOfficeRelationshipAgent(oldPartnerRel)
            --                  );
            --
            --             call revokePermissionFromRole(
            --                     findPermissionId(partnerDetailsUuid, 'UPDATE'),
            --                     hsOfficeRelationshipAdmin(oldPartnerRel)
            --                  );
            --
            --             call revokePermissionFromRole(
            --                     findPermissionId(partnerDetailsUuid, 'DELETE'),
            --                     hsOfficeRelationshipOwner(oldPartnerRel)
            --                  );

            -- Grants for PartnerDetails

            call grantPermissionsToRole(
                    getRoleId(hsOfficeRelationshipOwner(newPartnerRel)),
                    array[findPermissionId(partnerDetailsUuid, 'DELETE')]
                 );

            call grantPermissionsToRole(
                    getRoleId(hsOfficeRelationshipAdmin(newPartnerRel)),
                    array[findPermissionId(partnerDetailsUuid, 'UPDATE')]
                 );

            call grantPermissionsToRole(
                -- Yes, here hsOfficePartnerAGENT is used, not hsOfficePartnerTENANT.
                -- Do NOT grant view permission on partner-details to hsOfficeRelationshipTENANT!
                -- Otherwise package-admins etc. would be able to read the data.
                    getRoleId(hsOfficeRelationshipAgent(newPartnerRel)),
                    array[findPermissionId(partnerDetailsUuid, 'SELECT')]
                 );

        end if;

    else
        raise exception 'invalid usage of TRIGGER';
    end if;

    call leaveTriggerForObjectUuid(partnerUuid);
    return NEW;
end; $$;

/*
    An AFTER INSERT TRIGGER which creates the role structure for a new customer.
 */
create trigger createRbacRolesForHsOfficePartner_Trigger
    after insert
    on hs_office_partner
    for each row
execute procedure hsOfficePartnerRbacRolesTrigger();

/*
    An AFTER UPDATE TRIGGER which updates the role structure of a customer.
 */
create trigger updateRbacRolesForHsOfficePartner_Trigger
    after update
    on hs_office_partner
    for each row
execute procedure hsOfficePartnerRbacRolesTrigger();
--//


-- ============================================================================
--changeset hs-office-partner-rbac-IDENTITY-VIEW:1 endDelimiter:--//
-- ----------------------------------------------------------------------------
call generateRbacIdentityViewFromProjection('hs_office_partner', $idName$
    partnerNumber || ':' ||
    (select idName
            from hs_office_person_iv p
            left join hs_office_relationship r on r.uuid = target.partnerRoleUuid
            where p.uuid = r.relHolderUuid)
    || '-' ||
    (select idName
            from hs_office_contact_iv c
            left join hs_office_relationship r on r.uuid = target.partnerRoleUuid
            where c.uuid = r.contactUuid)
    $idName$);
--//


-- ============================================================================
--changeset hs-office-partner-rbac-RESTRICTED-VIEW:1 endDelimiter:--//
-- ----------------------------------------------------------------------------
call generateRbacRestrictedView('hs_office_partner',
    'target.partnerNumber',
    $updates$
        partnerRoleUuid = new.partnerRoleUuid
    $updates$);
--//


-- ============================================================================
--changeset hs-office-partner-rbac-NEW-PARTNER:1 endDelimiter:--//
-- ----------------------------------------------------------------------------
/*
    Creates a global permission for new-partner and assigns it to the Hostsharing admins role.
 */
do language plpgsql $$
    declare
        addCustomerPermissions uuid[];
        globalObjectUuid       uuid;
        globalAdminRoleUuid    uuid ;
    begin
        call defineContext('granting global new-partner permission to global admin role', null, null, null);

        globalAdminRoleUuid := findRoleId(globalAdmin());
        globalObjectUuid := (select uuid from global);
        addCustomerPermissions := createPermissions(globalObjectUuid, array ['new-partner']);
        call grantPermissionsToRole(globalAdminRoleUuid, addCustomerPermissions);
    end;
$$;

/**
    Used by the trigger to prevent the add-customer to current user respectively assumed roles.
 */
create or replace function addHsOfficePartnerNotAllowedForCurrentSubjects()
    returns trigger
    language PLPGSQL
as $$
begin
    raise exception '[403] new-partner not permitted for %',
        array_to_string(currentSubjects(), ';', 'null');
end; $$;

/**
    Checks if the user or assumed roles are allowed to create a new customer.
 */
create trigger hs_office_partner_insert_trigger
    before insert
    on hs_office_partner
    for each row
    -- TODO.spec: who is allowed to create new partners
    when ( not hasAssumedRole() )
execute procedure addHsOfficePartnerNotAllowedForCurrentSubjects();
--//

